@@ -129,6 +129,7 @@ struct Singleton end
 
 @testset "StructTypes" begin
 
+@test StructTypes.StructType(Union{}) == StructTypes.SingletonType()
 @test StructTypes.StructType(Union{Int, Missing}) == StructTypes.Struct()
 @test StructTypes.StructType(Any) == StructTypes.Struct()
 @test StructTypes.StructType(A) == StructTypes.UnorderedStruct()
@@ -200,6 +201,7 @@ y = DictWrapper(x)
 
 @test StructTypes.construct(Fruit, :apple) == apple
 @test StructTypes.construct(Fruit, "apple") == apple
+@test_throws ArgumentError StructTypes.construct(Fruit, "grapple")
 x = "apple"
 @test StructTypes.construct(Fruit, pointer(x), 5) == apple
 
@@ -207,6 +209,7 @@ x = "apple"
 @test StructTypes.construct(Symbol, pointer(x), 5) == :apple
 x = "499beb72-22ea-11ea-3366-55749430b981"
 @test StructTypes.construct(UUID, pointer(x), 36) == UUID(x)
+@test StructTypes.construct(UUID, pointer(x), 36; dateformat=dateformat"mm-dd-yyyy") == UUID(x)
 @test StructTypes.construct(Date, "11-30-2019"; dateformat=dateformat"mm-dd-yyyy") == Date(2019, 11, 30)
 
 @test StructTypes.StructType(UInt8) == StructTypes.NumberType()
@@ -720,6 +723,12 @@ StructTypes.defaults(::Type{C2}) = (b=2.5,)
             @test StructTypes.constructfrom(a, c) == c
         end
     end
+    @testset "constructfrom with Union" begin
+        dict = Dict(:a=>0, :b=>2.0, :c=>"c")
+        b = StructTypes.constructfrom(StructTypes.Struct(), Union{B, LotsOfFields}, dict)
+        @test typeof(b) == B
+        b = StructTypes.constructfrom(StructTypes.Struct(), Union{LotsOfFields, B}, dict)
+    end
     @testset "constructfrom with subtypeclosure" begin
         dict = Dict(:type => ("Road", 2, 10), :make => "Spesh", :model => "Roubaix", :year=>2020)
         bike = StructTypes.constructfrom(Bicycle, dict)
@@ -815,3 +824,39 @@ StructTypes.defaults(::Type{Defaultable}) = (b="b",)
     @test StructTypes.constructfrom(Defaultable, Dict(:a=>"a")) == Defaultable("a", "b")
     @test StructTypes.constructfrom(Defaultable, (a="a", b="c")) == Defaultable("a", "c")
 end
+
+# CustomStruct subtypes
+abstract type Vehicle2 end
+
+struct Car2 <: Vehicle2
+    make::String
+    model::String
+    seatingCapacity::Int
+    topSpeed::Float64
+end
+
+struct Truck2 <: Vehicle2
+    # type::String no need of this
+    make::String
+    model::String
+    payloadCapacity::Float64
+end
+
+StructTypes.StructType(::Type{Vehicle2}) = StructTypes.AbstractType()
+StructTypes.subtypekey(::Type{Vehicle2}) = :type
+StructTypes.subtypes(::Type{Vehicle2}) = (car=Car2, truck=Truck2)
+StructTypes.@register_struct_subtype Vehicle2 Car2
+StructTypes.@register_struct_subtype Vehicle2 Truck2
+
+@testset "register_struct_subtype" begin
+    nt = (; :type => :car, :make => "Mercedes-Benz", :model => "S500", :seatingCapacity => 5, :topSpeed => 250.1)
+    car = Car2(nt)
+    @test StructTypes.lower(car) === nt
+    @test StructTypes.lowertype(Car2) === typeof(nt)
+    @test typeof(car) == Car2
+    @test car.make == "Mercedes-Benz"
+end
+
+@testset "struct type on complex" begin
+    @test StructTypes.StructType(ComplexF64) == StructTypes.Struct()
+end
